import os
import stat

from polyglot.core.extension import Extensions
from polyglot.core.result import Result
from polyglot.core.display import Display

from polyglot.exceptions.exceptions import (
    PolyglotFileNotFoundError
)

class Polyglot(object):
    """
    The main polyglot class. An instance of this class
    is created by the user to use the core functions of
    the module

    Attributes:
        ignore -- The files to ignore
        directory -- The abspath of the directory to check
        files -- All the files inside of the directory

    """

    def __init__(self, directory_name: str, ignore=[]):
        assert isinstance(ignore, list), "Expected to be a list"
        self.ignore = ignore
        self.directory = Polyglot.find_directory_path(directory_name)
        self.files = self.__find_directory_files(self.directory)

    @staticmethod
    def find_directory_path(directory_path: str):
        """
        Determine the directory path based on
        the parameter. If the path is a dot(.) return the
        current working directory, else return 
        the path if the path is a directory, else
        throw an error
        """
        assert isinstance(directory_path, str), "Path expected to be a string"
        if directory_path == ".":
            return os.getcwd()

        if os.path.isdir(directory_path):
            return directory_path

        raise PolyglotFileNotFoundError(f"{directory_path} does not exist")

    def __find_hidden_files(self, hidden, filepath):
        """
        Make sure that the root of the file
        is not hidden
        """
        hidden_root = [
            str(filepath).startswith(hidden_file) for hidden_file in hidden
        ]
        return True in hidden_root

    def __find_directory_files(self, directory):
        """
        Find all the files by walking through
        the directory tree
        """
        filenames = []
        hidden_directories = []
        for (root, dirs, files) in os.walk(directory, topdown=True):
            if Polyglot.is_hidden_directory(root):
                hidden_directories.append(root)

            if not self.__find_hidden_files(hidden_directories, root):
                for filename in files:
                    if filename in self.ignore:
                        continue
                    filenames.append(os.path.join(root, filename))

        return filenames

    @staticmethod
    def is_hidden_directory(filepath):
        assert isinstance(filepath, str), "Expected a string"
        return bool(
            os.stat(filepath).st_file_attributes & stat.FILE_ATTRIBUTE_HIDDEN)

    def show(self, language_detection_file=None, display=False):
        extensions = Extensions(language_detection_file, display, self.files)
        data = extensions.get_extension_data()

        result = Result(data).show_file_information()
        if display:
            display_text = Display(result)
        return result


import os
import requests
import yaml

LANGUAGE_FILE = "https://raw.githubusercontent.com/github/linguist/master/lib/linguist/languages.yml"


def validate_argument_types(values, types, message):
    assert len(values) == len(
        types), "Values and types should have the same length"
    for index in range(len(values)):
        assert isinstance(values[index], types[index]), str(message)
    return True


def install_files(read_url, write_file_dir, filename, extension):
    assert isinstance(read_url, str), "Read url expected to be a string"
    assert isinstance(write_file_dir,
                      str), "Write path expected to be a string"

    filename = os.path.join(write_file_dir, f"{filename}.{extension}")
    try:
        with open(filename, "wb") as file_writer:
            file_writer.write(
                requests.get(read_url, allow_redirects=True).content)
    except Exception as exception:
        raise Exception

    return filename


class Extensions(object):
    def __init__(self, language_file, display, files):

        self.language_detection_file = language_file
        self.display_output = display
        self.filenames = files

        self.languages = {}

        self.content = self.remove_unwanted_keys(
            self.__create_language_file(self.language_detection_file)[0])

    def get_extension_data(self):
        return self.__split_files(self.filenames, self.content)

    def __split_files(self, files, content):
        """
        Loop through each file in the files array
        and determine the language with the help of
        the language extension 
        """
        for filename in files:
            language = self.__find_language_name(filename, content)
            if language not in self.languages:
                self.languages[language] = []

            self.languages[language].append(filename)
        return self.languages

    def __find_language_name(self, filename, content):
        extension = f".{filename.split('.')[-1]}"
        for language_key in content:
            if "extensions" not in content[language_key]:
                continue

            if extension in content[language_key]["extensions"]:
                return language_key
        return "Unknown file"

    def remove_unwanted_keys(self, file_content):
        """
        Remove all the unwanted keys from the 
        file_content dictionary and only keep
        the 'extensions' key
        """
        for language in dict(file_content):
            assert isinstance(file_content[language], dict), "Expected a dict"
            for key in dict(file_content[language]):
                if key != "extensions":
                    del file_content[language][key]
        return file_content

    def __create_language_file(self, language_file):
        """
        If language file is mentioned, and the file is a string
        return the filecontent and the number of lines

        Else, install the language file from the internet
        and return the file_content along with the number of lines
        """
        if language_file is not None and isinstance(language_file, str):
            if not language_file.endswith(".yml"):
                raise Exception("Language file expected to be a yaml file")
            return Extensions.read_file_data(language_file, True)

        return Extensions.read_file_data(
            install_files(LANGUAGE_FILE, os.getcwd(), "language", "yml"), True)

    @staticmethod
    def read_file_data(filename, is_yaml=False):
        """
        Read the specified filename and if the file
        is a yaml file,parse the yaml string
        using the yaml library
        """
        with open(filename, "r") as file_reader:
            file_content = file_reader.read()
            line_number_count = len(file_content.split("\n"))

            if not is_yaml:
                return file_content, line_number_count

            return yaml.safe_load(file_content), line_number_count



import sys as sys
import logging as logging
import time as time
from clint.textui import colored


def stop_current_application(exit_reason=None, set_timeout=0):
    assert isinstance(set_timeout, int), "Expected an integer"
    if exit_reason is not None:
        logging.error(f"Exiting application [{exit_reason}]")

    time.sleep(set_timeout)
    sys.exit()


class PolyglotException(object):
    def __init__(self,
                 error_message,
                 suggestion=None,
                 timeout=None,
                 fatal=True):
        assert self.is_valid_timeout(
            timeout), "Timeout expected to be an integer"
        assert isinstance(fatal, bool), "Fatal expected a boolean value"

        self.error_message = str(error_message)
        self.error_is_fatal = fatal

        self.suggestion = suggestion
        self.create_exception_message(timeout)

    def create_exception_message(self, timeout=None):
        if timeout is not None: time.sleep(timeout)

        self.suggestion = self.create_suggestion_message(self.suggestion)
        self.throw_exception(self.error_message, self.suggestion)

    def throw_exception(self, error, suggestion):
        throw_exception_data = [
            colored.red(f"ERROR: {error}"),
            colored.green(suggestion)
        ]
        for element in throw_exception_data:
            print(element)

        if self.error_is_fatal:
            stop_current_application()

    def is_valid_timeout(self, timeout):
        return isinstance(timeout, int) or timeout == None

    def create_suggestion_message(self, suggestion):
        if suggestion == None: return None

        return f"HELP: {suggestion}"

    def __len__(self):
        return len(str(self.error_message))

    def __str__(self):
        return str(self.error_message)

    def __int__(self):
        return self.__len__()

    def __bool__(self):
        return bool(self.error_is_fatal)


import sys
import os
import json

from polyglot.arguments.position import Position
from polyglot.exceptions.custom import PolyglotException
from polyglot.core.polyglot import Polyglot


class Arguments(object):
    """
    Pass a set of arguments which is parsed and
    later executed

    Attributes-
        arguments -- The arguments to parse
        return_value -- Whether to return any value
        position -- The lexer postion

    """
    def __init__(self, arguments=None, return_value=False):
        self.arguments = sys.argv[1:] if not arguments else arguments
        self.return_value = return_value

        self.position = Position(0)

    def parse(self):
        """
        Parse the arguments and validate
        them . Also execute the functions
        """
        assert isinstance(self.arguments, list)
        valid_flags = ["--dir", "--o", "--show", "--ignore"]

        parameters = {
            "dir": os.getcwd(),
            "o": None,
            "show": True,
            "ignore": []
        }

        current_character = self.position.current_character(self.arguments)
        while current_character is not None:
            if not self.is_valid_flag(valid_flags, current_character):
                exception = PolyglotException(
                    f"{current_character} is not recogonised as a valid parmeter",
                    f"Try again with valid parameters",
                    fatal=False)
                return None
            character_key = current_character[2:]
            if "=" not in character_key:
                exception = PolyglotException(
                    f"User equal-to(=) to add value to parameters",
                    f"Try again with valid values",
                    fatal=False)
                return None

            if character_key.count("=") > 1:
                exception = PolyglotException(
                    f"More than one assignments for the same parameter",
                    f"Try again with valid values",
                    fatal=False)
                return None

            data = character_key.split("=")
            key, value = data[0], data[1]

            parameters[key] = value

            self.position.increment()
            current_character = self.position.current_character(self.arguments)

        return_data = self.validate_parameters(parameters)
        if return_data == None:
            return return_data
        else:
            polyglot = Polyglot(parameters["dir"], ignore=parameters["ignore"])
            data = polyglot.show(display=parameters["show"])

            if parameters["o"]:
                with open(parameters["o"], "w") as writer:
                    writer.write(json.dumps(data))

            if self.return_value:
                return data

    def is_valid_flag(self, valid_cases, current_character):
        for valid_case_index in range(len(valid_cases)):
            if current_character.startswith(valid_cases[valid_case_index]):
                return True

        return False

    def validate_parameters(self, parameters):
        if parameters["show"] not in [str(True), str(False)]:
            exception = PolyglotException("Invalid value for paramter show",
                                          "Try again",
                                          fatal=False)
            return None

        parameters["show"] = bool(parameters["show"])
        parameters["ignore"] = parameters["ignore"].split(",")
        return parameters


from prettytable import PrettyTable
from clint.textui import colored


class Display(object):
    def __init__(self, display_text):
        assert isinstance(display_text, dict), "Expected a dict"
        self.text = display_text

        if not "files" in self.text or not "lines" in self.text:
            raise NameError("Cannot find required keys - lines, files")

        self.display_output()

    def display_output(self):
        """
        Verify the dict elements to be dicts 
        and then continue to print out
        the dict data in tabular form using
        prettytable.
        """
        self.verify_text()

        for display_text_type in self.text:
            print("\n")
            table = PrettyTable()
            table.field_names = ["Language", display_text_type]
            for data in self.text[display_text_type]:
                table.add_rows([[data, self.text[display_text_type][data]]])
            print(colored.yellow(table))

    def verify_text(self):
        assert isinstance(self.text["files"],
                          dict), "Files expected to be a dict"
        assert isinstance(self.text["lines"],
                          dict), "lines expected to be a dict"



import os


class Result(object):
    def __init__(self, file_information):
        assert isinstance(file_information, dict), "Expected a dict"

        self.file_information = file_information
        self.data = {"files": {}, "lines": {}}

    def show_file_information(self):
        files = self.__find_by_files(self.file_information)
        lines = self.__find_by_lines(self.file_information)

        return self.data

    def __find_by_files(self, data):
        length = sum([len(data[key]) for key in data])
        for file_type in data:
            self.data["files"][file_type] = round(
                (len(data[file_type]) / length) * 100, 2)

    def __find_by_lines(self, data):
        lines = {}
        for file_type in data:
            file_line_count = 0
            for filename in data[file_type]:
                with open(filename, "r") as line_counter:
                    file_line_count += len(line_counter.read().split("\n"))
            lines[file_type] = file_line_count
        total_lines = sum([lines[key] for key in lines])
        for line_key in data:
            self.data["lines"][line_key] = round(
                (lines[line_key] / total_lines) * 100, 2)



class Position(object):
    def __init__(self, initial_position):
        assert isinstance(initial_position, int), "Expected an integer"
        self.position = initial_position

    def increment(self, increment_by=1):
        self.position += 1
        return self.position

    def decrement(self, decrement_by=1):
        self.position -= 1
        return self.position

    def current_character(self, data, increment_value=False):
        assert isinstance(data, list), "Data expected to be a string"
        if len(data) == self.position:
            return None

        return_value = data[self.position]
        if increment_value:
            self.increment()

        return return_value



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

DEPENDENCIES = ["requests", "pyyaml", "prettytable", "clint"]

setuptools.setup(
    name="python-polyglot",  # Replace with your own username
    version="2.0.0",
    author="P Pranav Baburaj",
    author_email="code-roller@googlegroups.com",
    description=
    "Find the percentage of programming languages used in your project",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pranavbaburaj/polyglot",
    packages=setuptools.find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    install_requires=DEPENDENCIES,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


from clint.textui import colored


class PolyglotFileNotFoundError(FileNotFoundError):
    def __init__(self, message):
        self.message = colored.red(message)

        super().__init__(self.message)

    def str(self):
        return self.message
